@@ -4,11 +4,19 @@ import os
 import geopandas as gpd
 import numpy as np
 import pandas as pd
-from shapely.wkb import dumps
-from shapely.wkb import loads
 from tqdm import tqdm
 
 
+def ext(filepath):
+    '''扩展名'''
+    return os.path.splitext(filepath)[1]
+
+
+def fn(filepath):
+    '''路径及文件名（不含扩展名）'''
+    return os.path.splitext(filepath)[0]
+
+
 def max_grid():
     '''防止单个单元格文件过大而报错'''
     import sys
@@ -31,18 +39,18 @@ def rdf(filepath: str) -> pd.DataFrame:
     :return: dataframe
     '''
     max_grid()
-    if os.path.splitext(filepath)[1] == '.csv':
+    if ext(filepath) == '.csv':
         try:
             df = pd.read_csv(filepath, engine='python', encoding='utf-8-sig')
         except:
             df = pd.read_csv(filepath, engine='python')
-    elif os.path.splitext(filepath)[1] == '.xls':
+    elif ext(filepath) == '.xls':
         df = pd.read_excel(filepath)
-    elif os.path.splitext(filepath)[1] == '.xlsx':
+    elif ext(filepath) == '.xlsx':
         df = pd.read_excel(filepath)
-    elif os.path.splitext(filepath)[1] == '.shp':
+    elif ext(filepath) == '.shp':
         try:
-            df = gpd.GeoDataFrame.from_file(filepath, encoding='utf-8-sig')
+            df = gpd.GeoDataFrame.from_file(filepath)
         except UnicodeEncodeError:
             df = gpd.GeoDataFrame.from_file(filepath, encoding='GBK')
     else:
@@ -60,18 +68,17 @@ def to_csv_by_line(filename: str, data: list):
         csv_write.writerow(data)
 
 
-def rename2lnglat(df) -> pd.DataFrame:
+def ensure_lnglat(df) -> pd.DataFrame:
     '''将df中的经纬度重命名为lng和lat'''
-    col_dict = {'经度': 'lng', '纬度': 'lat', 'lon': 'lng', 'lng_WGS': 'lng', 'lat_WGS': 'lat', 'lon_WGS': 'lng',
-                'longitude': 'lng', 'latitude': 'lat', "geom": "geometry"}
-    df = df.rename(columns=col_dict)
+    from ricco import to_lnglat_dict
+    df.rename(columns=to_lnglat_dict, inplace=True)
     return df
 
 
 def read_and_rename(file: str) -> pd.DataFrame:
     '''读取文件并将经纬度统一为lng和lat，并按照经纬度排序'''
     df = rdf(file)
-    df = rename2lnglat(df)
+    df = ensure_lnglat(df)
     if 'lat' in df.columns:
         df.sort_values(['lat', 'lng'], inplace=True)
         df = df.reset_index(drop=True)
@@ -108,9 +115,9 @@ def mkdir_2(path: str):
         os.makedirs(path)
 
 
-def split_csv(filename: str, n=5):
+def split_csv(filename: str, n: int = 5, encoding: str = 'utf-8'):
     '''将文件拆分为多个同名文件，放置在与文件同名文件夹下的不同Part_文件夹中'''
-    dir_name = os.path.splitext(os.path.basename(filename))[0]
+    dir_name = fn(os.path.basename(filename))
     abs_path = os.getcwd()
     df = rdf(filename)
     t = len(df)
@@ -126,12 +133,15 @@ def split_csv(filename: str, n=5):
             add = df.iloc[low:, :]
         else:
             add = df.iloc[low: high, :]
-        add.to_csv(savefile, index=0, encoding='utf-8')
+        add.to_csv(savefile, index=0, encoding=encoding)
 
 
 def valid_check(polygon_geom):
     '''检验面的有效性'''
+    from shapely.wkb import loads
     df = polygon_geom.copy()
+    if len(df[df['geometry'].isna()]) > 0:
+        raise ValueError('geometry中有空值，请检查')
     df['geometry'] = df['geometry'].apply(lambda x: loads(x, hex=True))
     df = gpd.GeoDataFrame(df)
     df.crs = 'epsg:4326'
@@ -142,13 +152,34 @@ def valid_check(polygon_geom):
         raise Exception('有效性检验失败，请检查并修复面')
 
 
-def shp2csv(shpfile_name: str):
+def _dumps(x, hex=True, srid=4326):
+    from shapely.wkb import dumps
+    try:
+        x = dumps(x, hex=hex, srid=srid)
+    except AttributeError:
+        x = None
+    return x
+
+
+def shp2csv(shpfile_name: str, encoding='utf-8'):
     '''shapefile 转 csv 文件'''
     df = rdf(shpfile_name)
-    df['geometry'] = df['geometry'].apply(lambda x: dumps(x, hex=True, srid=4326))
+    print(df.head())
+    df = gpd.GeoDataFrame(df)
+    df['geometry'] = df['geometry'].apply(lambda x: _dumps(x, hex=True, srid=4326))
     df.crs = 'epsg:4326'
-    save_path = os.path.splitext(shpfile_name)[0] + '.csv'
-    df.to_csv(save_path, encoding='utf-8-sig', index=False)
+    save_path = fn(shpfile_name) + '.csv'
+    print(df.head())
+    df.to_csv(save_path, encoding=encoding, index=False)
+
+
+def _loads(x, hex=True):
+    from shapely.wkb import loads
+    try:
+        x = loads(x, hex=hex)
+    except AttributeError:
+        x = None
+    return x
 
 
 def csv2shp(filename: str):
@@ -158,9 +189,9 @@ def csv2shp(filename: str):
     df = df.rename(columns={'名称': 'name',
                             'geom': 'geometry'})
     df = gpd.GeoDataFrame(df)
-    df['geometry'] = df['geometry'].apply(lambda x: loads(x, hex=True))
+    df['geometry'] = df['geometry'].apply(lambda x: _loads(x, hex=True))
     df.crs = 'epsg:4326'
-    save_path = os.path.splitext(filename)[0] + '.shp'
+    save_path = fn(filename) + '.shp'
     try:
         df.to_file(save_path, encoding='utf-8')
     except fiona.errors.SchemaError:
@@ -188,15 +219,16 @@ def extract_num(string: str,
 
     :param string: 输入的字符串
     :param num_type:  输出的数字类型，int/float/str，默认为str
-    :param method: 结果计算方法，对结果列表求最大/最小/平均/和/等，numpy方法，默认返回列表本身
+    :param method: 结果计算方法，对结果列表求最大/最小/平均/和等，numpy方法，默认返回列表本身
     :param join_list: 是否合并列表，默认FALSE
     :param ignore_pct: 是否忽略百分号，默认True
     :return:
     '''
     import re
-    import numpy
     from warnings import warn
 
+    import numpy
+
     string = str(string)
     if ignore_pct:
         lis = re.findall(r"\d+\.?\d*", string)
@@ -252,12 +284,12 @@ def ensure_list(val):
     return [val]
 
 
-def segment(x, gap, sep: str = '-', unit: str = '') -> str:
+def segment(x, gap: (list, float, int), sep: str = '-', unit: str = '') -> str:
     '''
     区间段划分工具
 
     :param x: 数值
-    :param gap: 间隔
+    :param gap: 间隔，固定间隔或列表
     :param unit: 单位，末尾
     :param sep: 分隔符，中间
     :return: 区间段 'num1分隔符num2单位'：‘80-100米’
@@ -268,7 +300,10 @@ def segment(x, gap, sep: str = '-', unit: str = '') -> str:
             if x >= lis[i]:
                 return lis[i], lis[i + 1]
 
-    if isinstance(gap, list):
+    x = to_float(x)
+    if (x is None) | (x == np.nan) | (x == '') | (x == float('nan')):
+        return ''
+    elif isinstance(gap, list):
         gap = sorted(list(set(gap)))
         if x < gap[0]:
             return '%d%s以下' % (gap[0], unit)
@@ -278,7 +313,7 @@ def segment(x, gap, sep: str = '-', unit: str = '') -> str:
             l = between_list(x, gap)[0]
             h = between_list(x, gap)[1]
             s = '%d%s%d%s' % (l, sep, h, unit)
-    else:
+    elif isinstance(gap, (int, float)):
         if x >= 0:
             l = int(x / gap) * gap
             h = l + gap
@@ -287,6 +322,8 @@ def segment(x, gap, sep: str = '-', unit: str = '') -> str:
             l = int(x / gap) * gap
             h = l - gap
             s = '%d%s%d%s' % (h, sep, l, unit)
+    else:
+        raise TypeError('gap参数数据类型错误')
     return s
 
 
@@ -302,3 +339,61 @@ def standard(serise: (pd.Series, list),
     serise[serise >= 100] = 100
     serise[serise <= min_score] = min_score
     return serise
+
+
+def col_round(df, col: list):
+    '''对整列进行四舍五入，默认绝对值大于1的数值保留两位小数，小于1 的保留4位'''
+
+    def _round(x):
+        if abs(x) >= 1:
+            return round(x, 2)
+        else:
+            return round(x, 4)
+
+    col = ensure_list(col)
+    for i in col:
+        df[i] = df[i].apply(lambda x: _round(x))
+    return df
+
+
+def fuzz_match(string: str, ss: (list, pd.Series)):
+    '''
+    为某一字符串从某一集合中匹配相似度最高的元素
+
+    :param x: 输入的字符串
+    :param ss: 要去匹配的集合
+    :return: 字符串及相似度组成的列表
+    '''
+    from fuzzywuzzy import fuzz
+
+    def _ratio(s, x):
+        return fuzz.ratio(s, x), fuzz.partial_ratio(s, x)
+
+    max_r, max_pr, max_s = 0, 0, None
+    for s in ss:
+        r, pr = _ratio(s, string)
+        if r > max_r:
+            max_r = r
+            max_pr = pr
+            max_s = s
+    return max_s, max_r, max_pr
+
+
+def fuzz_df(df: pd.DataFrame,
+            col: str,
+            target_serise: (list, pd.Series)) -> pd.DataFrame:
+    '''
+    为DataFrame中的某一列，从某个集合中匹配相似度最高的元素
+
+    :param df: 输入的dataframe
+    :param col: 要匹配的列
+    :param target_serise: 从何处匹配， list/pd.Serise
+    :return:
+    '''
+    df[[col + '_target',
+        'normal_score',
+        'partial_score']] = df.apply(lambda x:
+                                     fuzz_match(x[col], target_serise),
+                                     result_type='expand',
+                                     axis=1)
+    return df
